import assert from "assert";
import { isNonNullType, isScalarType } from "graphql";
function getArg(name, directive) {
  assert(
    directive.arguments,
    `Expected @${directive} directive to have arguments`
  );
  const arg = directive.arguments?.find((a) => a.name.value === name);
  assert(arg, `${name} argument is required`);
  return arg;
}
function getOptionalArg(name, directive) {
  return directive.arguments?.find((arg) => arg.name.value === name);
}
function getArgStringValue(fieldName, directive) {
  const prefixArg = getArg(fieldName, directive);
  assert(
    prefixArg.value.kind === "StringValue",
    `Expected @${directive.name.value} directive argument "${fieldName}" to be a string, but got ${prefixArg.value.kind}`
  );
  return prefixArg.value.value;
}
function getArgEnumValue(fieldName, directive) {
  const prefixArg = getArg(fieldName, directive);
  assert(
    prefixArg.value.kind === "EnumValue",
    `Expected @${directive.name.value} directive argument "${fieldName}" to be an enum, but got ${prefixArg.value.kind}`
  );
  return prefixArg.value.value;
}
function getArgStringArrayValue(fieldName, directive) {
  const arg = getArg(fieldName, directive);
  assert(arg.value.kind === "ListValue", `Expected ${fieldName} to be a list`);
  return arg.value.values.map((v) => {
    assert(
      v.kind === "StringValue",
      `Expected @${directive.name.value} directive argument "${fieldName}" to be a list of strings`
    );
    return v.value;
  });
}
function getOptionalArgBooleanValue(fieldName, directive) {
  const prefixArg = getOptionalArg(fieldName, directive);
  if (!prefixArg) {
    return void 0;
  }
  assert(
    prefixArg.value.kind === "BooleanValue",
    `Expected @${directive.name.value} directive argument "${fieldName}" to be a boolean, but got ${prefixArg.value.kind}`
  );
  return Boolean(prefixArg.value.value);
}
function getOptionalArgStringArrayValue(fieldName, directive) {
  const arg = getOptionalArg(fieldName, directive);
  if (!arg) {
    return void 0;
  }
  assert(arg.value.kind === "ListValue", `Expected ${fieldName} to be a list`);
  return arg.value.values.map((v) => {
    assert(
      v.kind === "StringValue",
      `Expected @${directive.name.value} directive argument "${fieldName}" to be a list of strings`
    );
    return v.value;
  });
}
function getOptionalArgStringValue(fieldName, directive) {
  const prefixArg = getOptionalArg(fieldName, directive);
  if (!prefixArg) {
    return "";
  }
  assert(
    prefixArg.value.kind === "StringValue",
    `Expected @${directive.name.value} directive argument "${fieldName}" to be a string, but got ${prefixArg.value.kind}`
  );
  return prefixArg.value.value;
}
function getDirective(name, nodeOrType) {
  if ("getFields" in nodeOrType) {
    assert(nodeOrType.astNode, "Expected type to have an AST node");
    nodeOrType = nodeOrType.astNode;
  }
  const directive = nodeOrType.directives?.find((d) => d.name.value === name);
  assert(
    directive,
    `Expected field ${nodeOrType.name.value} to have an @${name} directive`
  );
  return directive;
}
function getOptionalDirective(name, nodeOrType) {
  if ("getFields" in nodeOrType) {
    assert(nodeOrType.astNode, "Expected type to have an AST node");
    nodeOrType = nodeOrType.astNode;
  }
  if ("astNode" in nodeOrType) {
    assert(nodeOrType.astNode);
    return nodeOrType.astNode.directives?.find((d) => d.name.value === name);
  }
  return nodeOrType.directives?.find((d) => d.name.value === name);
}
function hasDirective(directiveName, objType) {
  return !!objType.astNode?.directives?.map(({ name }) => name.value).includes(directiveName);
}
function hasInterface(interfaceName, objType) {
  return !!objType.astNode?.interfaces?.map(({ name }) => name.value).includes(interfaceName);
}
function isType(typeName, fieldType) {
  let { type } = fieldType;
  if (isNonNullType(type)) {
    type = type.ofType;
  }
  return isScalarType(type) && type.name === typeName;
}
function filterNull(x) {
  return Boolean(x);
}
function getOptionalArgObjectValue(fieldName, directive) {
  const arg = getOptionalArg(fieldName, directive);
  if (!arg) {
    return void 0;
  }
  assert(
    arg.value.kind === "ObjectValue",
    `Expected @${directive.name.value} directive argument "${fieldName}" to be an object, but got ${arg.value.kind}`
  );
  return coerceObject(arg.value);
}
function coerceObject(value) {
  assert(value.kind === "ObjectValue", "Expected an ObjectValue");
  return Object.fromEntries(
    value.fields.map(({ name: { value: name }, value: v }) => [
      name,
      coerceValue(v)
    ])
  );
}
function coerceValue(value) {
  switch (value.kind) {
    case "StringValue":
      return value.value;
    case "BooleanValue":
      return value.value;
    case "IntValue":
      return parseInt(value.value, 10);
    case "FloatValue":
      return parseFloat(value.value);
    case "EnumValue":
      return value.value;
    case "ListValue":
      return value.values.map(coerceValue);
    case "ObjectValue":
      break;
    case "NullValue":
      break;
  }
}
export {
  filterNull,
  getArg,
  getArgEnumValue,
  getArgStringArrayValue,
  getArgStringValue,
  getDirective,
  getOptionalArg,
  getOptionalArgBooleanValue,
  getOptionalArgObjectValue,
  getOptionalArgStringArrayValue,
  getOptionalArgStringValue,
  getOptionalDirective,
  hasDirective,
  hasInterface,
  isType
};
