import assert from "assert";
import { isNonNullType, isScalarType } from "graphql";
import { camelCase, snakeCase } from "lodash";
import { extractChangeDataCaptureConfig } from "./extractors/cdc";
import { extractSecondaryIndexes } from "./extractors/indexes";
import { extractPrimaryKey } from "./extractors/primary-key";
import { extractTTLConfig } from "./extractors/ttl";
import {
  getArgStringValue,
  getDirective,
  getOptionalArg,
  getOptionalArgBooleanValue,
  getOptionalDirective,
  hasDirective,
  hasInterface,
  isType
} from "./helpers";
import { extractTable } from "./tables";
const models = /* @__PURE__ */ new Map();
function getModel(type) {
  const model = models.get(type);
  assert(model, `Model for type ${type} not found`);
  return model;
}
function extractModel(config, schema, type, outputPath) {
  const cachedModel = models.get(type);
  if (cachedModel) {
    return cachedModel;
  }
  const fields = extractFields(type);
  const fieldMap = Object.fromEntries(
    fields.map((field) => [field.fieldName, field])
  );
  const model = {
    changeDataCaptureConfig: extractChangeDataCaptureConfig(
      config,
      schema,
      type,
      outputPath
    ),
    consistent: hasDirective("consistent", type),
    fields,
    isLedger: hasDirective("ledger", type),
    isPublic: hasInterface("PublicModel", type),
    primaryKey: extractPrimaryKey(type, fieldMap),
    secondaryIndexes: extractSecondaryIndexes(config, type, fieldMap),
    ttlConfig: extractTTLConfig(type),
    typeName: type.name,
    ...extractTableInfo(type)
  };
  models.set(type, model);
  const wholeModel = model;
  const table = extractTable(config, schema, type, outputPath);
  wholeModel.table = table;
  return wholeModel;
}
function extractFields(type) {
  const fields = type.getFields();
  return Object.keys(fields).map((fieldName) => {
    const field = fields[fieldName];
    const computed = getOptionalDirective("computed", field);
    const importDetails = computed ? {
      importName: getArgStringValue("importName", computed),
      importPath: getArgStringValue("importPath", computed),
      isVirtual: !!getOptionalArgBooleanValue("virtual", computed)
    } : void 0;
    return {
      columnName: getAliasForField(field, type, fieldName),
      columnNamesForRead: getReadAliasesForField(field, type, fieldName),
      computeFunction: importDetails,
      ean: `:${fieldName}`,
      eav: `#${fieldName}`,
      fieldName,
      isDateType: isType("Date", field),
      isRequired: isNonNullType(field.type),
      isScalarType: isNonNullType(field.type) ? isScalarType(field.type.ofType) : isScalarType(field.type),
      typeName: isNonNullType(field.type) ? String(field.type.ofType) : field.type.name
    };
  });
}
function extractTableInfo(type) {
  const tableDirective = getOptionalDirective("table", type);
  return {
    enablePointInTimeRecovery: tableDirective ? getOptionalArgBooleanValue(
      "enablePointInTimeRecovery",
      tableDirective
    ) !== false : true
  };
}
function getAliasForField(field, type, fieldName) {
  if (hasDirective("ttl", field)) {
    return "ttl";
  }
  if (hasDirective("alias", field)) {
    const { astNode } = field;
    assert(astNode);
    return getArgStringValue("name", getDirective("alias", astNode));
  }
  switch (field.name) {
    case "version":
      return "_v";
    case "createdAt":
      return "_ct";
    case "updatedAt":
      return "_md";
    case "publicId":
      return "publicId";
    default:
      return getCaseType(type) === "CAMEL_CASE" ? camelCase(fieldName) : snakeCase(fieldName);
  }
}
function getReadAliasesForField(field, type, fieldName) {
  if (hasDirective("ttl", field)) {
    return ["ttl"];
  }
  if (hasDirective("alias", field)) {
    const { astNode } = field;
    assert(astNode);
    return [getArgStringValue("name", getDirective("alias", astNode))];
  }
  switch (field.name) {
    case "version":
      return ["_v"];
    case "createdAt":
      return ["_ct"];
    case "updatedAt":
      return ["_md"];
    case "publicId":
      return ["publicId"];
    default:
      return getCaseType(type) === "CAMEL_CASE" ? [camelCase(fieldName), snakeCase(fieldName)] : [snakeCase(fieldName), camelCase(fieldName)];
  }
}
function getCaseType(type) {
  const tableDirective = getOptionalDirective("table", type);
  if (tableDirective) {
    const arg = getOptionalArg("columnCase", tableDirective);
    if (arg) {
      assert(arg.value.kind === "EnumValue");
      if (arg.value.value === "CAMEL_CASE") {
        return "CAMEL_CASE";
      }
      assert(arg.value.value === "SNAKE_CASE");
    }
  }
  return "SNAKE_CASE";
}
export {
  extractModel,
  getModel
};
